@@ -1,6 +1,6 @@
 #####################################################################
 ##
-## $Id: hpd.R,v 1.7.2.1 2006/10/02 19:18:53 byandell Exp $
+## $Id: hpd.R,v 1.7.2.2 2006/10/23 17:00:37 byandell Exp $
 ##
 ##     Copyright (C) 2003 Brian S. Yandell
 ##
@@ -152,12 +152,10 @@ summary.qb.hpdone <- function(object, chr = chrs, digits = 3, ...)
     tmp <- cbind(tmp, tmp2, tmp3)
     tmprow <- dimnames(tmp3)[[2]]
   }
-  dimnames(tmp) <- list(attr(object$profile, "geno.names")[chr.hpd],
+  dimnames(tmp) <- list(attr(object$profile, "geno.names")[chr],
                         c(dimnames(object$hpd.region)[[2]],
                           attr(object, "profile"),
                           tmprow))
-
-  
   round(tmp, digits)
 }
 ###################################################################
